@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2012 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -22,8 +22,6 @@ import cups
 import gobject
 import os
 
-import asyncipp
-import asyncpk1
 import config
 from debug import *
 import debug
@@ -84,18 +82,25 @@ class _AsyncMethodCall:
         kwds['reply_handler'] = self.reply_handler
         kwds['error_handler'] = self.error_handler
         kwds['auth_handler'] = self.auth_handler
+        debugprint ("%s: calling %s" % (self, self._fn))
         self._fn (*args, **kwds)
 
     def reply_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to reply_handler at %s" % (self,
+                                                        self._reply_handler))
             self._reply_handler (self, self._reply_data, *args)
 
     def error_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to error_handler at %s" % (self,
+                                                        self._error_handler))
             self._error_handler (self, self._error_data, *args)
 
     def auth_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to auth_handler at %s" % (self,
+                                                       self._auth_handler))
             self._auth_handler (self, self.auth_data, *args)
 
 ######
@@ -128,6 +133,7 @@ class Connection(SemanticOperations):
 
         if use_pk and try_as_root:
             debugprint ("Using polkit-1 connection class")
+            import asyncpk1
             c = asyncpk1.PK1Connection (reply_handler=subst_reply_handler,
                                         error_handler=subst_error_handler,
                                         host=host, port=port,
@@ -136,6 +142,7 @@ class Connection(SemanticOperations):
             self._conn = c
         else:
             debugprint ("Using IPP connection class")
+            import asyncipp
             c = asyncipp.IPPAuthConnection (reply_handler=subst_reply_handler,
                                             error_handler=subst_error_handler,
                                             auth_handler=subst_auth_handler,
@@ -199,6 +206,7 @@ class Connection(SemanticOperations):
             del self._methodcalls[i]
             args = args[1:]
         if reply_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, reply_handler))
             reply_handler (self, *args)
 
     def _subst_error_handler (self, methodcall, error_handler, *args):
@@ -208,6 +216,7 @@ class Connection(SemanticOperations):
             del self._methodcalls[i]
             args = args[1:]
         if error_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, error_handler))
             error_handler (self, *args)
 
     def _subst_auth_handler (self, methodcall, auth_handler, prompt, method, resource):
@@ -216,6 +225,7 @@ class Connection(SemanticOperations):
             i = self._methodcalls.index (methodcall)
             del self._methodcalls[i]
         if auth_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, auth_handler))
             auth_handler (prompt, self, method, resource)
 
     def set_auth_info (self, password):
