@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
@@ -145,7 +145,7 @@ class _IPPConnectionThread(threading.Thread):
                 debugprint ("...success")
                 self._reply (result)
             except Exception, e:
-                debugprint ("...failure")
+                debugprint ("...failure (%s)" % repr (e))
                 self._error (e)
 
             self._queue.task_done ()
@@ -345,11 +345,14 @@ class _IPPAuthOperation:
             return self._reconnect_error (conn, exc)
 
         if self._cancel:
+            debugprint ("%s (_error_handler): canceled so chaining up" % self)
             return self._error (exc)
 
         if self._reconnect:
             self._reconnect = False
             self._reconnected = True
+            debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                        (self, self_user))
             conn.reconnect (self._user,
                             reply_handler=self._reconnect_reply,
                             error_handler=self._reconnect_error)
@@ -377,6 +380,10 @@ class _IPPAuthOperation:
             return self._error (exc)
 
         # Not authorized.
+        if forbidden:
+            debugprint ("%s (_error_handler): forbidden" % self)
+        else:
+            debugprint ("%s (_error_handler): not authorized" % self)
 
         if (self._try_as_root and
             self._user != 'root' and
@@ -407,6 +414,8 @@ class _IPPAuthOperation:
         authconn.global_authinfocache.remove_auth_info (host=host,
                                                         port=port)
         self._use_password = ''
+        debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                    (self, self._user))
         conn.reconnect (self._user,
                         reply_handler=self._reconnect_reply,
                         error_handler=self._reconnect_error)
@@ -563,21 +572,31 @@ class _IPPAuthOperation:
                        _("Retry"), gtk.RESPONSE_OK)
         d.set_default_response (gtk.RESPONSE_OK)
         d.connect ("response", self._on_retry_server_error_response)
+        debugprint ("%s (_reconnect_error): presenting error dialog (%s; %s)" %
+                    (self, msg, message))
         d.show ()
 
     def _on_retry_server_error_response (self, dialog, response):
         dialog.destroy ()
         if response == gtk.RESPONSE_OK:
+            debugprint ("%s: got retry response, reconnecting (as %s)..." %
+                        (self, self._conn.thread.user))
             self._conn.reconnect (self._conn.thread.user,
                                   reply_handler=self._reconnect_reply,
                                   error_handler=self._reconnect_error)
         else:
+            debugprint ("%s: got cancel response" % self)
             self._error (cups.IPPError (0, _("Operation canceled")))
 
     def _error (self, exc):
+        debugprint ("%s (_error): handling %s" % (self, repr (exc)))
         if self._client_error_handler:
+            debugprint ("%s (_error): calling %s" %
+                        (self, self._client_error_handler))
             self._client_error_handler (self._conn, exc)
             self._destroy ()
+        else:
+            debugprint ("%s (_error): no client error handler set" % self)
 
 ###
 ### The user-visible class.
