@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -21,7 +21,10 @@
 import cups
 import dbus
 import gobject
-import gtk
+try:
+    import gtk
+except:
+    pass
 import os
 import sys
 import tempfile
@@ -75,6 +78,7 @@ class _PK1AsyncMethodCall:
         pk_method = proxy.get_dbus_method (self._pk_method_name)
 
         try:
+            debugprint ("%s: calling %s" % (self, pk_method))
             pk_method (*self._pk_args,
                         reply_handler=self._pk_reply_handler,
                         error_handler=self._pk_error_handler,
@@ -102,9 +106,17 @@ class _PK1AsyncMethodCall:
             return
 
         if str (error) == '':
-            gtk.gdk.threads_enter ()
+            try:
+                gtk.gdk.threads_enter ()
+            except:
+                pass
+            debugprint ("%s: no error, calling reply handler %s" %
+                        (self, self._client_reply_handler))
             self._client_reply_handler (self._conn, self._unpack_fn (*args))
-            gtk.gdk.threads_leave ()
+            try:
+                gtk.gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
@@ -117,9 +129,17 @@ class _PK1AsyncMethodCall:
 
         if exc.get_dbus_name () == CUPS_PK_NEED_AUTH:
             exc = cups.IPPError (cups.IPP_NOT_AUTHORIZED, 'pkcancel')
-            gtk.gdk.threads_enter ()
+            try:
+                gtk.gdk.threads_enter ()
+            except:
+                pass
+            debugprint ("%s: no auth, calling error handler %s" %
+                        (self, self._client_error_handler))
             self._client_error_handler (self._conn, exc)
-            gtk.gdk.threads_leave ()
+            try:
+                gtk.gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
@@ -131,12 +151,15 @@ class _PK1AsyncMethodCall:
         # Make the 'connection' parameter consistent with PK callbacks.
         self._fallback_kwds["reply_handler"] = self._ipp_reply_handler
         self._fallback_kwds["error_handler"] = self._ipp_error_handler
+        debugprint ("%s: calling %s" % (self, self._fallback_fn))
         self._fallback_fn (*self._fallback_args, **self._fallback_kwds)
 
     def _ipp_reply_handler (self, conn, *args):
         if self._destroyed:
             return
 
+        debugprint ("%s: chaining up to %s" % (self,
+                                               self._client_reply_handler))
         self._client_reply_handler (self._conn, *args)
         self._destroy ()
 
@@ -144,6 +167,8 @@ class _PK1AsyncMethodCall:
         if self._destroyed:
             return
 
+        debugprint ("%s: chaining up to %s" % (self,
+                                               self._client_error_handler))
         self._client_error_handler (self._conn, *args)
         self._destroy ()
 
