@@ -2,7 +2,7 @@
 
 ## check-device-ids
 
-## Copyright (C) 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -18,7 +18,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import dbus
 import cups
@@ -301,12 +301,29 @@ for device, attrs in devices.iteritems ():
             break
     if mdls:
         (s, bestmatches) = ppds._findBestMatchPPDs (mdls, mdl)
-        if s == ppds.STATUS_SUCCESS:
+        if s == ppds.FIT_EXACT:
             matches = matches.union (set (bestmatches))
 
     missing = set (matches) - set (drivers)
     for each in missing:
-        print "%s       MISSING  %s [%s]" % (more, each,
-                                             driver_uri_to_pkg (each))
+        try:
+            ppd_device_id = ppds.getInfoFromPPDName (each).get ('ppd-device-id')
+        except Exception, e:
+            print e
+            ppd_device_id = None
+
+        if ppd_device_id:
+            print "%s       WRONG    %s [%s]" % (more, each,
+                                                 driver_uri_to_pkg (each))
+            ppd_id_fields = cupshelpers.parseDeviceID (ppd_device_id)
+            for field in ["MFG", "MDL"]:
+                value = id_fields[field]
+                ppd_value = ppd_id_fields[field]
+                if value.lower () != ppd_value.lower ():
+                    print "%s                      %s:%s;" % (more, field, ppd_value)
+                    print "%s                should be:%s;" % (more, value)
+        else:
+            print "%s       MISSING  %s [%s]" % (more, each,
+                                                 driver_uri_to_pkg (each))
 
     i += 1
