@@ -1,6 +1,6 @@
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Red Hat, Inc.
 ## Authors:
 ##  Florian Festi <ffesti@redhat.com>
 ##  Tim Waugh <twaugh@redhat.com>
@@ -22,6 +22,7 @@
 import cups, pprint, os, tempfile, re, string
 import locale
 from . import _debugprint
+from . import config
 
 class Printer:
     _flags_blacklist = ["options", "local"]
@@ -562,18 +563,51 @@ class Device:
                 return -1
             if self.type == "parallel":
                 return 1
-            if other.type == "hp" or other.type == "hpfax":
+            if other.type == "hp":
                 return 1
-            if self.type == "hp" or self.type == "hpfax":
+            if self.type == "hp":
+                return -1
+            if other.type == "hpfax":
+                return 1
+            if self.type == "hpfax":
                 return -1
             if other.type == "dnssd":
                 return 1
             if self.type == "dnssd":
                 return -1
+            if other.type == "socket":
+                return 1
+            if self.type == "socket":
+                return -1
+            if other.type == "lpd":
+                return 1
+            if self.type == "lpd":
+                return -1
+            if other.type == "ipps":
+                return 1
+            if self.type == "ipps":
+                return -1
+            if other.type == "ipp":
+                return 1
+            if self.type == "ipp":
+                return -1
             if other.type == "usb":
                 return 1
             if self.type == "usb":
                 return -1
+        if self.type == "dnssd" and other.type == "dnssd":
+            if other.uri.find("._pdl-datastream") != -1: # Socket
+                return 1
+            if self.uri.find("._pdl-datastream") != -1:
+                return -1
+            if other.uri.find("._printer") != -1: # LPD
+                return 1
+            if self.uri.find("._printer") != -1:
+                return -1
+            if other.uri.find("._ipp") != -1: # IPP
+                return 1
+            if self.uri.find("._ipp") != -1:
+                return -1
         result = cmp(bool(self.id), bool(other.id))
         if not result:
             result = cmp(self.info, other.info)
@@ -811,10 +845,10 @@ def missingExecutables(ppd):
                     continue
 
                 exepath = pathcheck (exe,
-                                     "/usr/lib/cups/filter:"
+                                     config.cupsserverbindir + "/filter:"
                                      "/usr/lib64/cups/filter")
                 if not exepath:
-                    add_missing ("/usr/lib/cups/filter/" + exe)
+                    add_missing (config.cupsserverbindir + "/filter/" + exe)
 
     return exes_to_install
 
