@@ -19,7 +19,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import gobject
 import gtk
@@ -38,7 +38,7 @@ class GtkGUI(gobject.GObject):
             if domain:
                 bld.set_translation_domain (domain)
 
-            bld.add_from_file (os.path.join (ui_dir, xmlfile + ".glade"))
+            bld.add_from_file (os.path.join (ui_dir, xmlfile + ".ui"))
             for name in names:
                 widget = bld.get_object(name)
                 if widget is None:
