@@ -1,5 +1,5 @@
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 ##  Jiri Popelka <jpopelka@redhat.com>
@@ -958,6 +958,9 @@ class JobViewer (GtkGUI):
                     except gnomekeyring.DeniedError:
                         debugprint ("gnomekeyring: denied for %s" %
                                     keyring_attrs)
+                    except gnomekeyring.CancelledError:
+                        debugprint ("gnomekeyring: cancelled for %s" %
+                                    keyring_attrs)
                     except Exception, e:
                         debugprint ("gnomekeyring: caught exception %s" % e)
 
@@ -1243,15 +1246,14 @@ class JobViewer (GtkGUI):
         self.set_statusicon_visibility ()
 
     def on_icon_configure_printers_activate(self, menuitem):
-        if self.loop:
-            env = {}
-            for name, value in os.environ.iteritems ():
-                if name == "SYSTEM_CONFIG_PRINTER_UI":
-                    continue
-                env[name] = value
-            p = subprocess.Popen ([ "system-config-printer" ],
-                                  close_fds=True, env=env)
-            gobject.timeout_add_seconds (10, self.poll_subprocess, p)
+        env = {}
+        for name, value in os.environ.iteritems ():
+            if name == "SYSTEM_CONFIG_PRINTER_UI":
+                continue
+            env[name] = value
+        p = subprocess.Popen ([ "system-config-printer" ],
+                              close_fds=True, env=env)
+        gobject.timeout_add_seconds (10, self.poll_subprocess, p)
 
     def poll_subprocess(self, process):
         returncode = process.poll ()
@@ -1375,7 +1377,7 @@ class JobViewer (GtkGUI):
             try:
                 attrs=c.getJobAttributes(jobid)
                 printer_uri=attrs['job-printer-uri']
-                document_count=attrs['document-count']
+                document_count=attrs.get ('document-count', 0)
                 for document_number in range(1, document_count+1):
                     document=c.getDocument(printer_uri, jobid, document_number)
                     tempfile = document.get('file')
@@ -2271,17 +2273,20 @@ class JobViewer (GtkGUI):
             icon = self.icon_jobs
 
         if s == cups.IPP_JOB_HELD:
-            theme = gtk.icon_theme_get_default ()
-            emblem = theme.load_icon (gtk.STOCK_MEDIA_PAUSE, 22 / 2, 0)
-            copy = icon.copy ()
-            emblem.composite (copy, 0, 0,
-                              copy.get_width (),
-                              copy.get_height (),
-                              copy.get_width () / 2 - 1,
-                              copy.get_height () / 2 - 1,
-                              1.0, 1.0,
-                              gtk.gdk.INTERP_NEAREST, 255)
-            icon = copy
+            try:
+                theme = gtk.icon_theme_get_default ()
+                emblem = theme.load_icon (gtk.STOCK_MEDIA_PAUSE, 22 / 2, 0)
+                copy = icon.copy ()
+                emblem.composite (copy, 0, 0,
+                                  copy.get_width (),
+                                  copy.get_height (),
+                                  copy.get_width () / 2 - 1,
+                                  copy.get_height () / 2 - 1,
+                                  1.0, 1.0,
+                                  gtk.gdk.INTERP_NEAREST, 255)
+                icon = copy
+            except gobject.GError:
+                debugprint ("No %s icon available" % gtk.STOCK_MEDIA_PAUSE)
         else:
             # Check state reasons.
             printer = data['job-printer-name']
