@@ -2,7 +2,7 @@
 
 ## system-config-printer
 
-## Copyright (C) 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2013 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,7 @@ class PPDsLoader(gobject.GObject):
 
         self._ppdsmatch_result = None
         self._jockey_queried = False
+        self._jockey_has_answered = False
         self._local_cups = (self._host == None or
                             self._host == "localhost" or
                             self._host[0] == '/')
@@ -130,6 +131,7 @@ class PPDsLoader(gobject.GObject):
         return self._ppdsmatch_result
 
     def get_error (self):
+        debugprint ("%s: stored error is %s" % (self, repr (self._exc)))
         return self._exc
 
     def _dialog_response (self, dialog, response):
@@ -197,6 +199,7 @@ class PPDsLoader(gobject.GObject):
         conn.destroy ()
         self._conn = None
         self._ppds = None
+        self._exc = exc
         if self._dialog != None:
             self._dialog.destroy ()
             self._dialog = None
@@ -256,6 +259,7 @@ class PPDsLoader(gobject.GObject):
 
     def _jockey_reply (self, conn, result):
         debugprint ("Got Jockey result: %s" % repr (result))
+        self._jockey_has_answered = True
         try:
             self._installed_files = result[1]
         except:
