@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "gnu" -*-
  * udev-configure-printer - a udev callout to configure print queues
- * Copyright (C) 2009, 2010, 2011 Red Hat, Inc.
+ * Copyright (C) 2009, 2010, 2011, 2012 Red Hat, Inc.
  * Author: Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,13 @@
 /*
  * The protocol for this program is:
  *
- * udev-configure-printer add {DEVPATH}
- * udev-configure-printer remove {DEVPATH}
+ * udev-configure-printer add {DEVADDR}
+ * udev-configure-printer remove {DEVADDR}
  *
- * where DEVPATH is the path (%p) of the device
+ * where DEVADDR is one of:
+ *   the USB address of the device in the form usb-$env{BUSNUM}-$env{DEVNUM}
+ *   the device path of the device (%p)
+ *   the bluetooth address of the device
  */
 
 #define LIBUDEV_I_KNOW_THE_API_IS_SUBJECT_TO_CHANGE 1
@@ -45,13 +48,89 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <unistd.h>
-#include <usb.h>
+#include <libusb.h>
 #include <glib.h>
+#include <dirent.h>
 
 #define DISABLED_REASON "Unplugged or turned off"
 #define MATCH_ONLY_DISABLED 1
 #define USB_URI_MAP "/var/run/udev-configure-printer/usb-uris"
 
+#if (CUPS_VERSION_MAJOR > 1) || (CUPS_VERSION_MINOR > 5)
+#define HAVE_CUPS_1_6 1
+#endif
+
+/*
+ * CUPS 1.6 makes various structures private and
+ * introduces these ippGet and ippSet functions
+ * for all of the fields in these structures.
+ * http://www.cups.org/str.php?L3928
+ * We define (same signatures) our own accessors when CUPS < 1.6.
+ */
+#ifndef HAVE_CUPS_1_6
+const char *
+ippGetName(ipp_attribute_t *attr)
+{
+  return (attr->name);
+}
+
+ipp_op_t
+ippGetOperation(ipp_t *ipp)
+{
+  return (ipp->request.op.operation_id);
+}
+
+ipp_status_t
+ippGetStatusCode(ipp_t *ipp)
+{
+  return (ipp->request.status.status_code);
+}
+
+ipp_tag_t
+ippGetGroupTag(ipp_attribute_t *attr)
+{
+  return (attr->group_tag);
+}
+
+ipp_tag_t
+ippGetValueTag(ipp_attribute_t *attr)
+{
+  return (attr->value_tag);
+}
+
+int
+ippGetInteger(ipp_attribute_t *attr,
+              int             element)
+{
+  return (attr->values[element].integer);
+}
+
+const char *
+ippGetString(ipp_attribute_t *attr,
+             int             element,
+             const char      **language)
+{
+  return (attr->values[element].string.text);
+}
+
+ipp_attribute_t	*
+ippFirstAttribute(ipp_t *ipp)
+{
+  if (!ipp)
+    return (NULL);
+  return (ipp->current = ipp->attrs);
+}
+
+ipp_attribute_t *
+ippNextAttribute(ipp_t *ipp)
+{
+  if (!ipp || !ipp->current)
+    return (NULL);
+  return (ipp->current = ipp->current->next);
+}
+
+#endif
+
 struct device_uris
 {
   size_t n_uris;
@@ -441,147 +520,87 @@ parse_device_id (const char *device_id,
   free (fieldname);
 }
 
-static char *
-device_id_from_devpath (const char *devpath,
-			const struct usb_uri_map *map,
-			struct device_id *id,
-			char *usbserial, size_t usbseriallen,
-			char *usblpdev, size_t usblpdevlen)
+int
+device_file_filter(const struct dirent *entry)
 {
-  struct usb_uri_map_entry *entry;
-  struct udev *udev;
-  struct udev_device *dev, *parent_dev = NULL;
-  const char *sys;
-  const char *idVendorStr, *idProductStr, *serial;
-  char *end;
-  unsigned long idVendor, idProduct;
-  size_t syslen, devpathlen;
-  char *syspath;
-  struct usb_bus *bus;
-  struct usb_dev_handle *handle = NULL;
-  char ieee1284_id[1024];
-  const char *device_id = NULL;
-  int conf = 0, iface = 0;
-  int got = 0;
-  char *usb_device_devpath;
-  char *usblpdevpos, *dest;
+  return ((strstr(entry->d_name, "lp") != NULL) ? 1 : 0);
+}
 
-  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
+static char *
+get_ieee1284_id_from_child (struct udev *udev, struct udev_device *parent)
+{
+  struct udev_enumerate *udev_enum;
+  struct udev_list_entry *item, *first = NULL;
+  char *device_id = NULL;
 
-  udev = udev_new ();
-  if (udev == NULL)
+  udev_enum = udev_enumerate_new (udev);
+  if (!udev_enum)
     {
-      syslog (LOG_ERR, "udev_new failed");
+      syslog (LOG_ERR, "udev_enumerate_new failed");
       exit (1);
     }
 
-  /* For devices discovered via the usblp kernel module we read out the number
-   * of the /dev/usb/lp* device file, as there can be queues set up with 
-   * non-standard CUPS backends based on the /dev/usb/lp* device file and
-   * we want to avoid that an additional queue with a standard CUPS backend
-   * gets set up.
-   */
-  usblpdevpos = strstr(devpath, "/usb/lp");
-  if (usblpdevpos != NULL)
-    usblpdevpos += 7;
-  else
+  if (udev_enumerate_add_match_parent (udev_enum, parent) < 0)
     {
-      usblpdevpos = strstr(devpath, "/usblp");
-      if (usblpdevpos != NULL)
-	usblpdevpos += 6;
-    }
-  if (usblpdevpos != NULL)
-    {
-      for (dest = usblpdev;
-	   (*usblpdevpos >= '0') && (*usblpdevpos <= '9') &&
-	     (dest - usblpdev < usblpdevlen);
-	   usblpdevpos ++, dest ++)
-	*dest = *usblpdevpos;
-      *dest = '\0';
-    }
-
-  sys = udev_get_sys_path (udev);
-  syslen = strlen (sys);
-  devpathlen = strlen (devpath);
-  syspath = malloc (syslen + devpathlen + 1);
-  if (syspath == NULL)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "out of memory");
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "uname to add parent match");
       exit (1);
     }
 
-  memcpy (syspath, sys, syslen);
-  memcpy (syspath + syslen, devpath, devpathlen);
-  syspath[syslen + devpathlen] = '\0';
-
-  dev = udev_device_new_from_syspath (udev, syspath);
-  if (dev == NULL)
+  if (udev_enumerate_scan_devices (udev_enum) < 0)
     {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
+      exit (1);
+    }
+
+  first = udev_enumerate_get_list_entry (udev_enum);
+  udev_list_entry_foreach (item, first)
+    {
+      const char *ieee1284_id = NULL;
+      struct udev_device *dev;
+      dev = udev_device_new_from_syspath (udev,
+					  udev_list_entry_get_name (item));
+      if (dev == NULL)
+	continue;
+
+      ieee1284_id = udev_device_get_sysattr_value (dev, "ieee1284_id");
+      if (ieee1284_id)
+	device_id = g_strdup (ieee1284_id);
+
       udev_device_unref (dev);
-      udev_unref (udev);
-      syslog (LOG_ERR, "unable to access %s", syspath);
-      return NULL;
+      if (device_id)
+	break;
     }
 
-  parent_dev = udev_device_get_parent_with_subsystem_devtype (dev,
-							      "usb",
-							      "usb_device");
-  if (!parent_dev)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "Failed to get parent");
-      return NULL;
-    }
+  udev_enumerate_unref (udev_enum);
+  return device_id;
+}
 
-  usb_device_devpath = strdup (udev_device_get_devpath (parent_dev));
-  syslog (LOG_DEBUG, "device devpath is %s", usb_device_devpath);
+static char *
+get_ieee1284_id_using_libusb (struct udev_device *dev,
+			      const char *usbserial)
+{
+  const char *idVendorStr, *idProductStr;
+  unsigned long idVendor, idProduct;
+  char *end;
+  int conf = 0, iface = 0, altset = 0, numdevs = 0, i, n, m;
+  libusb_device **list;
+  struct libusb_device *device;
+  struct libusb_device_handle *handle = NULL;
+  struct libusb_device_descriptor devdesc;
+  struct libusb_config_descriptor *confptr = NULL;
+  const struct libusb_interface *ifaceptr = NULL;
+  const struct libusb_interface_descriptor *altptr = NULL;
+  char libusbserial[1024];
+  char ieee1284_id[1024];
+  int got = 0;
 
-  for (entry = map->entries; entry; entry = entry->next)
-    if (!strcmp (entry->devpath, usb_device_devpath))
-      break;
-
-  if (entry)
-    {
-      /* The map already had an entry so has already been dealt
-       * with.  This can happen because there are two "add"
-       * triggers: one for the usb_device device and the other for
-       * the usblp device.  We have most likely been triggered by
-       * the usblp device, so the usb_device rule got there before
-       * us and succeeded.
-       *
-       * Pretend we didn't find any device URIs that matched, and
-       * exit.
-       */
-      syslog (LOG_DEBUG, "Device already handled");
-      return NULL;
-    }
-
-  serial = udev_device_get_sysattr_value (parent_dev, "serial");
-  if (serial)
-    {
-      strncpy (usbserial, serial, usbseriallen);
-      usbserial[usbseriallen - 1] = '\0';
-    }
-  else
-    usbserial[0] = '\0';
-
-  /* See if we were triggered by a usblp add event. */
-  device_id = udev_device_get_sysattr_value (dev, "device/ieee1284_id");
-  if (device_id)
-    {
-      got = 1;
-      goto got_deviceid;
-    }
-
-  /* This is a low-level USB device.  Use libusb to fetch the Device ID. */
-  idVendorStr = udev_device_get_sysattr_value (parent_dev, "idVendor");
-  idProductStr = udev_device_get_sysattr_value (parent_dev, "idProduct");
+  idVendorStr = udev_device_get_sysattr_value (dev, "idVendor");
+  idProductStr = udev_device_get_sysattr_value (dev, "idProduct");
 
   if (!idVendorStr || !idProductStr)
     {
-      udev_device_unref (dev);
-      udev_unref (udev);
       syslog (LOG_ERR, "Missing sysattr %s",
 	      idVendorStr ?
 	      (idProductStr ? "serial" : "idProduct") : "idVendor");
@@ -605,110 +624,228 @@ device_id_from_devpath (const char *devpath,
   syslog (LOG_DEBUG, "Device vendor/product is %04zX:%04zX",
 	  idVendor, idProduct);
 
-  usb_init ();
-  usb_find_busses ();
-  usb_find_devices ();
-  for (bus = usb_get_busses (); bus && !got; bus = bus->next)
+  libusb_init(NULL);
+  numdevs = libusb_get_device_list(NULL, &list);
+  if (numdevs > 0)
+    for (i = 0; i < numdevs; i++)
+      {
+	device = list[i];
+
+	if (libusb_get_device_descriptor(device, &devdesc) < 0)
+	  continue;
+
+	if (!devdesc.bNumConfigurations || !devdesc.idVendor ||
+	    !devdesc.idProduct)
+	  continue;
+
+	if (devdesc.idVendor != idVendor || devdesc.idProduct != idProduct)
+	  continue;
+
+	for (conf = 0; conf < devdesc.bNumConfigurations; conf ++)
+	  {
+	    if (libusb_get_config_descriptor(device, conf, &confptr) < 0)
+	      continue;
+	    for (iface = 0, ifaceptr = confptr->interface;
+		 iface < confptr->bNumInterfaces;
+		 iface ++, ifaceptr ++)
+	      {
+		for (altset = 0, altptr = ifaceptr->altsetting;
+		     altset < ifaceptr->num_altsetting;
+		     altset ++, altptr ++)
+		  {
+		    if (altptr->bInterfaceClass != LIBUSB_CLASS_PRINTER ||
+			altptr->bInterfaceSubClass != 1)
+		      continue;
+		    
+		    if (libusb_open(device, &handle) < 0)
+		      {
+			syslog (LOG_DEBUG, "failed to open device");
+			continue;
+		      }
+
+		    if (usbserial[0] != '\0' &&
+			(libusb_get_string_descriptor_ascii(handle,
+						devdesc.iSerialNumber,
+						(unsigned char *)libusbserial,
+						sizeof(libusbserial))) > 0 &&
+			strcmp(usbserial, libusbserial) != 0)
+		      {
+			libusb_close (handle);
+			handle = NULL;
+			continue;
+		      }
+		      
+		    n = altptr->bInterfaceNumber;
+		    if (libusb_claim_interface(handle, n) < 0)
+		      {
+			libusb_close (handle);
+			handle = NULL;
+			syslog (LOG_DEBUG, "failed to claim interface");
+			continue;
+		      }
+		    if (n != 0 && libusb_claim_interface(handle, 0) < 0)
+		      {
+			syslog (LOG_DEBUG, "failed to claim interface 0");
+		      }
+		    
+		    m = altptr->bAlternateSetting;
+		    if (libusb_set_interface_alt_setting(handle, n, m)
+			< 0)
+		      {
+			libusb_close (handle);
+			handle = NULL;
+			syslog (LOG_DEBUG, "failed set altinterface");
+			continue;
+		      }
+
+		    memset (ieee1284_id, '\0', sizeof (ieee1284_id));
+		    if (libusb_control_transfer(handle,
+						LIBUSB_REQUEST_TYPE_CLASS |
+						LIBUSB_ENDPOINT_IN |
+						LIBUSB_RECIPIENT_INTERFACE,
+						0, conf,
+						(n << 8) | m,
+						(unsigned char *)ieee1284_id,
+						sizeof (ieee1284_id),
+						5000) < 0)
+		      {
+			libusb_close (handle);
+			handle = NULL;
+			syslog (LOG_ERR, "Failed to fetch Device ID");
+			continue;
+		      }
+
+		    got = 1;
+		    libusb_close (handle);
+		    break;
+		  }
+	      }
+	  }
+      }
+
+  libusb_free_device_list(list, 1);
+  libusb_exit(NULL);
+
+  if (got)
+    return g_strdup (ieee1284_id + 2);
+  return NULL;
+}
+
+static char *
+device_id_from_devpath (struct udev *udev, const char *devpath,
+			const struct usb_uri_map *map,
+			struct device_id *id,
+			char *usbserial, size_t usbseriallen,
+			char *usblpdev, size_t usblpdevlen)
+{
+  struct usb_uri_map_entry *entry;
+  struct udev_device *dev;
+  const char *serial;
+  size_t syslen, devpathlen;
+  char *syspath, *devicefilepath;
+  const char *device_id = NULL;
+  char *usb_device_devpath;
+  char *usblpdevpos, *dest;
+  struct dirent **namelist;
+  int num_names;
+
+  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
+
+  syslen = strlen ("/sys");
+  devpathlen = strlen (devpath);
+  syspath = malloc (syslen + devpathlen + 1);
+  if (syspath == NULL)
     {
-      struct usb_device *device;
-      for (device = bus->devices; device && !got; device = device->next)
+      syslog (LOG_ERR, "out of memory");
+      exit (1);
+    }
+  memcpy (syspath, "/sys", syslen);
+  memcpy (syspath + syslen, devpath, devpathlen);
+  syspath[syslen + devpathlen] = '\0';
+
+  devicefilepath = malloc (syslen + devpathlen + 5);
+  if (devicefilepath == NULL)
+    {
+      syslog (LOG_ERR, "out of memory");
+      exit (1);
+    }
+  memcpy (devicefilepath, syspath, syslen + devpathlen);
+  memcpy (devicefilepath + syslen + devpathlen, "/usb", 4);
+  devicefilepath[syslen + devpathlen + 4] = '\0';
+
+  /* For devices under control of the usblp kernel module we read out the number
+   * of the /dev/usb/lp* device file, as there can be queues set up with 
+   * non-standard CUPS backends based on the /dev/usb/lp* device file and
+   * we want to avoid that an additional queue with a standard CUPS backend
+   * gets set up.
+   */
+  num_names = scandir(devicefilepath, &namelist, device_file_filter, alphasort);
+  if (num_names <= 0)
+    num_names = scandir(syspath, &namelist, device_file_filter, alphasort);
+  if (num_names > 0)
+    {
+      usblpdevpos = strstr(namelist[0]->d_name, "lp");
+      if (usblpdevpos != NULL)
 	{
-	  struct usb_config_descriptor *confptr;
-	  if (device->descriptor.idVendor != idVendor ||
-	      device->descriptor.idProduct != idProduct ||
-	      !device->config)
-	    continue;
-
-	  conf = 0;
-	  for (confptr = device->config;
-	       conf < device->descriptor.bNumConfigurations && !got;
-	       conf++, confptr++)
-	    {
-	      struct usb_interface *ifaceptr;
-	      iface = 0;
-	      for (ifaceptr = confptr->interface;
-		   iface < confptr->bNumInterfaces && !got;
-		   iface++, ifaceptr++)
-		{
-		  struct usb_interface_descriptor *altptr;
-		  int altset = 0;
-		  for (altptr = ifaceptr->altsetting;
-		       altset < ifaceptr->num_altsetting && !got;
-		       altset++, altptr++)
-		    {
-		      if (altptr->bInterfaceClass == USB_CLASS_PRINTER &&
-			  altptr->bInterfaceSubClass == 1)
-			{
-			  int n;
-			  handle = usb_open (device);
-			  if (!handle)
-			    {
-			      syslog (LOG_DEBUG, "failed to open device");
-			      continue;
-			    }
-
-			  n = altptr->bInterfaceNumber;
-			  if (usb_claim_interface (handle, n) < 0)
-			    {
-			      usb_close (handle);
-			      handle = NULL;
-			      syslog (LOG_DEBUG, "failed to claim interface");
-			      continue;
-			    }
-
-			  if (n != 0 && usb_claim_interface (handle, 0) < 0)
-			    {
-			      usb_close (handle);
-			      handle = NULL;
-			      syslog (LOG_DEBUG, "failed to claim interface 0");
-			      continue;
-			    }
-
-			  n = altptr->bAlternateSetting;
-			  if (usb_set_altinterface (handle, n) < 0)
-			    {
-			      usb_close (handle);
-			      handle = NULL;
-			      syslog (LOG_DEBUG, "failed set altinterface");
-			      continue;
-			    }
-
-			  memset (ieee1284_id, '\0', sizeof (ieee1284_id));
-			  if (usb_control_msg (handle,
-					       USB_TYPE_CLASS |
-					       USB_ENDPOINT_IN |
-					       USB_RECIP_INTERFACE,
-					       0, conf, iface,
-					       ieee1284_id,
-					       sizeof (ieee1284_id),
-					       5000) < 0)
-			    {
-			      usb_close (handle);
-			      handle = NULL;
-			      syslog (LOG_ERR, "Failed to fetch Device ID");
-			      continue;
-			    }
-
-			  got = 1;
-			  usb_close (handle);
-			  break;
-			}
-		    }
-		}
-	    }
+	  usblpdevpos += 2;
+	  for (dest = usblpdev;
+	       (*usblpdevpos >= '0') && (*usblpdevpos <= '9') &&
+		 (dest - usblpdev < usblpdevlen);
+	       usblpdevpos ++, dest ++)
+	    *dest = *usblpdevpos;
+	  *dest = '\0';
 	}
     }
 
- got_deviceid:
-  if (got)
+  dev = udev_device_new_from_syspath (udev, syspath);
+  if (dev == NULL)
     {
-      if (!device_id)
-	device_id = ieee1284_id + 2;
-      parse_device_id (device_id, id);
+      udev_device_unref (dev);
+      syslog (LOG_ERR, "unable to access %s", syspath);
+      return NULL;
     }
 
+  usb_device_devpath = strdup (udev_device_get_devpath (dev));
+  syslog (LOG_DEBUG, "device devpath is %s", usb_device_devpath);
+
+  for (entry = map->entries; entry; entry = entry->next)
+    if (!strcmp (entry->devpath, usb_device_devpath))
+      break;
+
+  if (entry)
+    {
+      /* The map already had an entry so has already been dealt
+       * with.  This can happen because there are two "add"
+       * triggers: one for the usb_device device and the other for
+       * the usblp device.  We have most likely been triggered by
+       * the usblp device, so the usb_device rule got there before
+       * us and succeeded.
+       *
+       * Pretend we didn't find any device URIs that matched, and
+       * exit.
+       */
+      syslog (LOG_DEBUG, "Device already handled");
+      return NULL;
+    }
+
+  serial = udev_device_get_sysattr_value (dev, "serial");
+  if (serial)
+    {
+      strncpy (usbserial, serial, usbseriallen);
+      usbserial[usbseriallen - 1] = '\0';
+    }
+  else
+    usbserial[0] = '\0';
+
+  device_id = get_ieee1284_id_from_child (udev, dev);
+  if (!device_id)
+    /* Use libusb to fetch the Device ID. */
+    device_id = get_ieee1284_id_using_libusb (dev, usbserial);
+
+  if (device_id)
+    parse_device_id (device_id, id);
+
   udev_device_unref (dev);
-  udev_unref (udev);
   return usb_device_devpath;
 }
 
@@ -742,6 +879,70 @@ device_id_from_bluetooth (const char *bdaddr, struct device_id *id)
   g_free (device_id);
 }
 
+static char *
+devpath_from_usb_devaddr (struct udev *udev, const char *devaddr)
+{
+  char *devname_ending = g_strdup (devaddr);
+  char *devname;
+  const char *devpath;
+  struct udev_enumerate *udev_enum;
+  struct udev_list_entry *first = NULL;
+  struct udev_device *device;
+
+  g_strdelimit (devname_ending, "-", '/');
+  devname = g_strdup_printf("/dev/bus/%s", devname_ending);
+  g_free (devname_ending);
+
+  udev_enum = udev_enumerate_new (udev);
+  if (udev_enum == NULL)
+    {
+      syslog (LOG_ERR, "udev_enumerate_new failed");
+      exit (1);
+    }
+
+  if (udev_enumerate_add_match_property (udev_enum, "DEVNAME", devname) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_add_match_property failed");
+      exit (1);
+    }
+
+  if (udev_enumerate_scan_devices (udev_enum) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
+      exit (1);
+    }
+
+  first = udev_enumerate_get_list_entry (udev_enum);
+  if (first == NULL)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "no device named %s found", devname);
+      exit (1);
+    }
+
+  device = udev_device_new_from_syspath (udev,
+					 udev_list_entry_get_name (first));
+  if (device == NULL)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "unable to examine device");
+      exit (1);
+    }
+
+  devpath = udev_device_get_devpath (device);
+  udev_enumerate_unref (udev_enum);
+  if (!devpath)
+    {
+      syslog (LOG_ERR, "no devpath for device");
+      exit (1);
+    }
+
+  g_free (devname);
+  return g_strdup (devpath);
+}
+
 static char *
 uri_from_bdaddr (const char *devpath)
 {
@@ -769,15 +970,15 @@ cupsDoRequestOrDie (http_t *http,
   if (answer == NULL)
     {
       syslog (LOG_ERR, "failed to send IPP request %d",
-	      request->request.op.operation_id);
+	      ippGetOperation (request));
       exit (1);
     }
 
-  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+  if (ippGetStatusCode (answer) > IPP_OK_CONFLICT)
     {
       syslog (LOG_ERR, "IPP request %d failed (%d)",
-	      request->request.op.operation_id,
-	      answer->request.status.status_code);
+	      ippGetOperation (request),
+	      ippGetStatusCode (answer));
       exit (1);
     }
 
@@ -845,26 +1046,26 @@ find_matching_device_uris (struct device_id *id,
   answer = cupsDoRequestOrDie (cups, request, "/");
   httpClose (cups);
 
-  for (attr = answer->attrs; attr; attr = attr->next)
+  for (attr = ippFirstAttribute (answer); attr; attr = ippNextAttribute (answer))
     {
       const char *device_uri = NULL;
       struct device_id this_id;
       this_id.full_device_id = this_id.mfg = this_id.mdl = this_id.sern = NULL;
 
-      while (attr && attr->group_tag != IPP_TAG_PRINTER)
-	attr = attr->next;
+      while (attr && ippGetGroupTag (attr) != IPP_TAG_PRINTER)
+	attr = ippNextAttribute (answer);
 
       if (!attr)
 	break;
 
-      for (; attr && attr->group_tag == IPP_TAG_PRINTER; attr = attr->next)
+      for (; attr && ippGetGroupTag (attr) == IPP_TAG_PRINTER; attr = ippNextAttribute (answer))
 	{
-	  if (attr->value_tag == IPP_TAG_URI &&
-	      !strcmp (attr->name, "device-uri"))
-	    device_uri = attr->values[0].string.text;
-	  else if (attr->value_tag == IPP_TAG_TEXT &&
-		   !strcmp (attr->name, "device-id"))
-	    parse_device_id (attr->values[0].string.text, &this_id);
+	  if (ippGetValueTag (attr) == IPP_TAG_URI &&
+	      !strcmp (ippGetName (attr), "device-uri"))
+	    device_uri = ippGetString (attr, 0, NULL);
+	  else if (ippGetValueTag (attr) == IPP_TAG_TEXT &&
+		   !strcmp (ippGetName (attr), "device-id"))
+	    parse_device_id (ippGetString (attr, 0, NULL), &this_id);
 	}
 
       /* Only use device schemes in our preference order for matching
@@ -1158,6 +1359,7 @@ for_each_matching_queue (struct device_uris *device_uris,
     "printer-state-message",
   };
   char usblpdevstr1[32] = "", usblpdevstr2[32] = "";
+  int firstqueue = 1;
 
   if (cups == NULL)
     return 0;
@@ -1175,9 +1377,9 @@ for_each_matching_queue (struct device_uris *device_uris,
       exit (1);
     }
 
-  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+  if (ippGetStatusCode (answer) > IPP_OK_CONFLICT)
     {
-      if (answer->request.status.status_code == IPP_NOT_FOUND)
+      if (ippGetStatusCode (answer) == IPP_NOT_FOUND)
 	{
 	  /* No printer queues configured. */
 	  ippDelete (answer);
@@ -1185,7 +1387,7 @@ for_each_matching_queue (struct device_uris *device_uris,
 	}
 
       syslog (LOG_ERR, "CUPS-Get-Printers request failed (%d)",
-	      answer->request.status.status_code);
+	      ippGetStatusCode (answer));
       exit (1);
     }
 
@@ -1200,12 +1402,9 @@ for_each_matching_queue (struct device_uris *device_uris,
 	       usblpdev);
       snprintf(usblpdevstr2, sizeof(usblpdevstr2), "/usblp%s",
 	       usblpdev);
-      syslog (LOG_DEBUG,
-	      "Consider also queues with \"%s\" or \"%s\" in their URIs as matching",
-	      usblpdevstr1, usblpdevstr2);
     }
 
-  for (attr = answer->attrs; attr; attr = attr->next)
+  for (attr = ippFirstAttribute (answer); attr; attr = ippNextAttribute (answer))
     {
       const char *this_printer_uri = NULL;
       const char *this_device_uri = NULL;
@@ -1215,27 +1414,27 @@ for_each_matching_queue (struct device_uris *device_uris,
       char *this_device_uri_n, *device_uri_n;
       const char *ps1, *ps2, *pi1, *pi2;
 
-      while (attr && attr->group_tag != IPP_TAG_PRINTER)
-	attr = attr->next;
+      while (attr && ippGetGroupTag (attr) != IPP_TAG_PRINTER)
+	attr = ippNextAttribute (answer);
 
       if (!attr)
 	break;
 
-      for (; attr && attr->group_tag == IPP_TAG_PRINTER; attr = attr->next)
+      for (; attr && ippGetGroupTag (attr) == IPP_TAG_PRINTER; attr = ippNextAttribute (answer))
 	{
-	  if (attr->value_tag == IPP_TAG_URI)
+	  if (ippGetValueTag (attr) == IPP_TAG_URI)
 	    {
-	      if (!strcmp (attr->name, "device-uri"))
-		this_device_uri = attr->values[0].string.text;
-	      else if (!strcmp (attr->name, "printer-uri-supported"))
-		this_printer_uri = attr->values[0].string.text;
+	      if (!strcmp (ippGetName (attr), "device-uri"))
+		this_device_uri = ippGetString (attr, 0, NULL);
+	      else if (!strcmp (ippGetName (attr), "printer-uri-supported"))
+		this_printer_uri = ippGetString (attr, 0, NULL);
 	    }
-	  else if (attr->value_tag == IPP_TAG_TEXT &&
-		   !strcmp (attr->name, "printer-state-message"))
-	    printer_state_message = attr->values[0].string.text;
-	  else if (attr->value_tag == IPP_TAG_ENUM &&
-		   !strcmp (attr->name, "printer-state"))
-	    state = attr->values[0].integer;
+	  else if (ippGetValueTag (attr) == IPP_TAG_TEXT &&
+		   !strcmp (ippGetName (attr), "printer-state-message"))
+	    printer_state_message = ippGetString (attr, 0, NULL);
+	  else if (ippGetValueTag (attr) == IPP_TAG_ENUM &&
+		   !strcmp (ippGetName (attr), "printer-state"))
+	    state = ippGetInteger (attr, 0);
 	}
 
       if (!this_device_uri)
@@ -1244,8 +1443,6 @@ for_each_matching_queue (struct device_uris *device_uris,
 	goto skip;
 
       this_device_uri_n = normalize_device_uri(this_device_uri);
-      syslog (LOG_DEBUG, "URI of print queue: %s, normalized: %s",
-	      this_device_uri, this_device_uri_n);
       pi1 = strstr (this_device_uri, "interface=");
       ps1 = strstr (this_device_uri, "serial=");
       for (i = 0; i < device_uris->n_uris; i++)
@@ -1256,8 +1453,8 @@ for_each_matching_queue (struct device_uris *device_uris,
 	     level USB (libusb) we cannot simply compare URIs, must
 	     consider also URIs as equal if one has an "interface"
 	     or "serial" attribute and the other not. If both have
-	     the attribute it must naturally match. We check which attributes 
-             are there and this way determine up to which length the two URIs 
+	     the attribute it must naturally match. We check which attributes
+             are there and this way determine up to which length the two URIs
              must match. Here we can assume that if a URI has an "interface"
 	     attribute it has also a "serial" attribute, as this URI is
 	     an URI obtained via libusb and these always have a "serial"
@@ -1277,16 +1474,26 @@ for_each_matching_queue (struct device_uris *device_uris,
 	    l = strlen(this_device_uri);
 	  else
 	    l = strlen(device_uris->uri[i]);
-	  syslog (LOG_DEBUG, "URI of detected printer: %s, normalized: %s",
-		  device_uris->uri[i], device_uri_n);
+	  if (firstqueue == 1)
+	    {
+	      syslog (LOG_DEBUG, "URI of detected printer: %s, normalized: %s",
+		      device_uris->uri[i], device_uri_n);
+	      if (i == 0 && strlen(usblpdev) > 0)
+		syslog (LOG_DEBUG,
+			"Consider also queues with \"%s\" or \"%s\" in their URIs as matching",
+			usblpdevstr1, usblpdevstr2);
+	    }
+	  if (i == 0)
+	    syslog (LOG_DEBUG, "URI of print queue: %s, normalized: %s",
+		    this_device_uri, this_device_uri_n);
 	  if ((!strncmp (device_uris->uri[i], this_device_uri, l)) ||
-	      (strstr (device_uri_n, this_device_uri_n) == 
+	      (strstr (device_uri_n, this_device_uri_n) ==
 	       device_uri_n) ||
-	      (strstr (this_device_uri_n, device_uri_n) == 
+	      (strstr (this_device_uri_n, device_uri_n) ==
 	       this_device_uri_n) ||
 	      ((strlen(usblpdev) > 0) &&
 	       ((strstr (this_device_uri, usblpdevstr1) != NULL) ||
-	       (strstr (this_device_uri, usblpdevstr2) != NULL)))) 
+	       (strstr (this_device_uri, usblpdevstr2) != NULL))))
 	    {
 	      matched++;
 	      syslog (LOG_DEBUG, "Queue %s has matching device URI",
@@ -1297,10 +1504,13 @@ for_each_matching_queue (struct device_uris *device_uris,
 		  (flags & MATCH_ONLY_DISABLED) == 0)
 		{
 		  (*fn) (this_printer_uri, context);
+		  break;
 		}
 	    }
 	}
 
+      firstqueue = 0;
+
     skip:
       if (!attr)
 	break;
@@ -1332,7 +1542,7 @@ enable_queue (const char *printer_uri, void *context)
       return;
     }
 
-  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+  if (ippGetStatusCode (answer) > IPP_OK_CONFLICT)
     syslog (LOG_ERR, "IPP-Resume-Printer request failed");
   else
     syslog (LOG_INFO, "Re-enabled printer %s", printer_uri);
@@ -1374,52 +1584,44 @@ bluetooth_verify_address (const char *bdaddr)
 }
 
 static int
-do_add (const char *cmd, const char *devpath, int detach)
+do_add (const char *cmd, const char *devaddr)
 {
-  pid_t pid;
-  int f;
   struct device_id id;
   struct device_uris device_uris;
   struct usb_uri_map *map;
+  struct udev *udev;
+  char *devpath = NULL;
   char *usb_device_devpath = NULL;
   char usbserial[256];
   char usblpdev[8] = "";
   gboolean is_bluetooth;
 
-  if (detach && getenv ("DEBUG") == NULL)
-    {
-      if ((pid = fork ()) == -1)
-	syslog (LOG_ERR, "Failed to fork process");
-      else if (pid != 0)
-	/* Parent. */
-	exit (0);
+  syslog (LOG_DEBUG, "add %s", devaddr);
 
-      close (STDIN_FILENO);
-      close (STDOUT_FILENO);
-      close (STDERR_FILENO);
-      f = open ("/dev/null", O_RDWR);
-      if (f != STDIN_FILENO)
-	dup2 (f, STDIN_FILENO);
-      if (f != STDOUT_FILENO)
-	dup2 (f, STDOUT_FILENO);
-      if (f != STDERR_FILENO)
-	dup2 (f, STDERR_FILENO);
-
-      setsid ();
-    }
-
-  syslog (LOG_DEBUG, "add %s", devpath);
-
-  is_bluetooth = bluetooth_verify_address (devpath);
+  is_bluetooth = bluetooth_verify_address (devaddr);
 
   map = read_usb_uri_map ();
   if (is_bluetooth) {
     usbserial[0] = '\0';
-    device_id_from_bluetooth (devpath, &id);
+    device_id_from_bluetooth (devaddr, &id);
   } else {
-    usb_device_devpath = device_id_from_devpath (devpath, map, &id,
+    udev = udev_new ();
+    if (udev == NULL)
+      {
+	syslog (LOG_ERR, "udev_new failed");
+	exit (1);
+      }
+
+    if (!strncmp (devaddr, "usb-", 4))
+      devpath = devpath_from_usb_devaddr (udev, devaddr);
+    else
+      devpath = g_strdup (devaddr);
+
+    usb_device_devpath = device_id_from_devpath (udev, devpath, map, &id,
 						 usbserial, sizeof (usbserial),
 						 usblpdev, sizeof (usblpdev));
+    g_free (devpath);
+    udev_unref (udev);
   }
 
   if (!id.mfg || !id.mdl)
@@ -1519,7 +1721,7 @@ remove_queue (const char *printer_uri)
       return;
     }
 
-  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+  if (ippGetStatusCode (answer) > IPP_OK_CONFLICT)
     syslog (LOG_ERR, "IPP-Delete-Printer request failed");
   else
     syslog (LOG_INFO, "Deleted printer %s as the corresponding device "
@@ -1553,7 +1755,7 @@ disable_queue (const char *printer_uri, void *context)
       return;
     }
 
-  if (answer->request.status.status_code > IPP_OK_CONFLICT)
+  if (ippGetStatusCode (answer) > IPP_OK_CONFLICT)
     syslog (LOG_ERR, "IPP-Pause-Printer request failed");
   else
     syslog (LOG_INFO, "Disabled printer %s as the corresponding device "
@@ -1564,24 +1766,40 @@ disable_queue (const char *printer_uri, void *context)
 }
 
 static int
-do_remove (const char *devpath)
+do_remove (const char *devaddr)
 {
   struct usb_uri_map *map;
   struct usb_uri_map_entry *entry, **prev;
   struct device_uris *uris = NULL;
   char usblpdev[8] = "";
-  syslog (LOG_DEBUG, "remove %s", devpath);
+  gchar *devpath = NULL;
+  syslog (LOG_DEBUG, "remove %s", devaddr);
 
-  if (bluetooth_verify_address (devpath))
+  if (bluetooth_verify_address (devaddr))
     {
       char *device_uri;
 
-      device_uri = uri_from_bdaddr (devpath);
+      device_uri = uri_from_bdaddr (devaddr);
       remove_queue (devpath);
       g_free (device_uri);
       return 0;
     }
 
+  if (!strncmp (devaddr, "usb-", 4))
+    {
+      struct udev *udev = udev_new ();
+      if (udev == NULL)
+	{
+	  syslog (LOG_ERR, "udev_new failed");
+	  exit (1);
+	}
+
+      devpath = devpath_from_usb_devaddr (udev, devaddr);
+      udev_unref (udev);
+    }
+  else
+    devpath = g_strdup (devaddr);
+
   map = read_usb_uri_map ();
   prev = &map->entries;
   for (entry = map->entries; entry; entry = entry->next)
@@ -1605,76 +1823,10 @@ do_remove (const char *devpath)
     }
 
   free_usb_uri_map (map);
+  g_free (devpath);
   return 0;
 }
 
-static int
-do_enumerate (const char *argv0)
-{
-  struct udev *udev;
-  struct udev_enumerate *e;
-  struct udev_list_entry *item = NULL, *first = NULL;
-  int r = 0;
-
-  udev = udev_new ();
-  if (udev == NULL)
-    {
-      syslog (LOG_ERR, "udev_new failed");
-      exit (1);
-    }
-
-  e = udev_enumerate_new (udev);
-  if (e == NULL)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "udev_enumerate_new failed");
-      exit (1);
-    }
-
-  if (udev_enumerate_add_match_tag (e, "udev-configure-printer") < 0)
-    {
-      udev_unref (udev);
-      udev_enumerate_unref (e);
-      syslog (LOG_ERR, "udev_enumerate_add_match_tag failed");
-      exit (1);
-    }
-
-  if (udev_enumerate_scan_devices (e) < 0)
-    {
-      udev_unref (udev);
-      udev_enumerate_unref (e);
-      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
-      exit (1);
-    }
-
-  first = udev_enumerate_get_list_entry (e);
-  udev_list_entry_foreach (item, first)
-    {
-      int k;
-      const char *p;
-      struct udev_device *d;
-
-      d = udev_device_new_from_syspath (udev, udev_list_entry_get_name(item));
-      if (!d)
-        continue;
-
-      p = udev_device_get_devpath (d);
-      if (p)
-        {
-          k = do_add (argv0, p, 0);
-          if (k != 0)
-            r = k;
-        }
-
-      udev_device_unref (d);
-  }
-
-  udev_enumerate_unref (e);
-  udev_unref (udev);
-
-  return r;
-}
-
 int
 main (int argc, char **argv)
 {
@@ -1702,9 +1854,9 @@ main (int argc, char **argv)
   openlog ("udev-configure-printer", 0, LOG_LPR);
   cupsSetPasswordCB (no_password);
   if (add)
-    return do_add (argv[0], argv[2], 1);
+    return do_add (argv[0], argv[2]);
   if (enumerate)
-    return do_enumerate (argv[0]);
+    return 0; // no-op
 
   return do_remove (argv[2]);
 }
